// -*- C++ -*-
/*!
 * @file  ConsoleOut.cpp
 * @brief Console output component
 * $Date: 2007-10-09 07:33:08 $
 *
 * $Id: ConsoleOut.cpp 706 2008-04-16 00:36:15Z n-ando $
 */

#include "ConsoleOut.h"

// Module specification
// <rtc-template block="module_spec">
static const char* consoleout_spec[] =
  {
    "implementation_id", "ConsoleOut",
    "type_name",         "ConsoleOut",
    "description",       "Console output component",
    "version",           "1.0",
    "vendor",            "Noriaki Ando, AIST",
    "category",          "example",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    ""
  };
// </rtc-template>

ConsoleOut::ConsoleOut(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_inIn("in", m_in),
    
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  
  // Set OutPort buffer
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}

ConsoleOut::~ConsoleOut()
{
}


RTC::ReturnCode_t ConsoleOut::onInitialize()
{
  registerInPort("in", m_inIn);
  return RTC::RTC_OK;
}

RTC::ReturnCode_t ConsoleOut::onActivated(RTC::UniqueId ec_id)
{
  AutoConnect();
  return RTC::RTC_OK;
}

RTC::ReturnCode_t ConsoleOut::onExecute(RTC::UniqueId ec_id)
{
  std::cout << "Execute" << std::endl;
  if (m_inIn.isNew())
    {
      m_inIn.read();
      std::cout << "Received: " << m_in.data << std::endl;
      std::cout << "TimeStamp: " << m_in.tm.sec << "[s] ";
      std::cout << m_in.tm.nsec << "[ns]" << std::endl;
    }
  usleep(1000);

  return RTC::RTC_OK;
}


RTC::ReturnCode_t ConsoleOut::AutoConnect()
{

  int _argc(0);
  char** _argv(0);

  std::string subs_type;
  std::string period;
	subs_type = "Periodic";
	period = "1.0";
  
  std::cout << "Subscription Type: " << subs_type << std::endl;
  if (period != "")
    std::cout << "Period: " << period << " [Hz]" << std::endl;

  CORBA::ORB_var orb = CORBA::ORB_init(_argc, _argv);
  CorbaNaming naming(orb, "127.0.0.1:9876");

  CorbaConsumer<RTC::RTObject,RTC::RTObject::_ptr_type> conin,conout;
  PortList_var pin;
  PortList_var pout;

  // find ConsoleOut0 component
  conout.setObject(naming.resolve("ConsoleOut0.rtc"));

  // get ports
  pout = conout->get_ports();
  pout[0]->disconnect_all();
  //  assert(pout->length() > 0);

  // find ConsoleIn0 component
  conin.setObject(naming.resolve("ConsoleIn0.rtc"));

  // get ports
  pin = conin->get_ports();
  pin[0]->disconnect_all();

  // connect ports
  ConnectorProfile prof;
  prof.connector_id = "";
  prof.name = CORBA::string_dup("connector0");
  prof.ports.length(2);
  prof.ports[0] = pin[0];
  prof.ports[1] = pout[0];
  CORBA_SeqUtil::push_back(
		prof.properties,
		NVUtil::newNV("dataport.interface_type","CORBA_Any")
	);
  CORBA_SeqUtil::push_back(
		prof.properties,
		NVUtil::newNV("dataport.dataflow_type","Push")
	);
  CORBA_SeqUtil::push_back(
		prof.properties,
		NVUtil::newNV("dataport.subscription_type", subs_type.c_str() )
	);
  if (period != "")
  CORBA_SeqUtil::push_back(prof.properties,
			   NVUtil::newNV("dataport.push_interval",period.c_str() )
	);

  ReturnCode_t ret;
  ret = pin[0]->connect(prof);
  assert(ret == RTC::RTC_OK);

  if(ret == RTC::RTC_OK)
    {
      std::cout << "Connector ID: " << prof.connector_id << std::endl;
      NVUtil::dump(prof.properties);
    }
  orb->destroy();

  usleep(100);

  return ret;

}

extern "C"
{
 
  void ConsoleOutInit(RTC::Manager* manager)
  {
    RTC::Properties profile(consoleout_spec);
    manager->registerFactory(profile,
                             RTC::Create<ConsoleOut>,
                             RTC::Delete<ConsoleOut>);
  }
  
};


